@@ -218,8 +218,21 @@ configuration file '~a': ~s")
                      (wrap-operation (sleep-operation timeout)
                                      (const 'timeout)))))
 
+(define (socket-monitor socket-file on-deletion)
+  "Return a thunk that monitors @var{socket-file} and calls @var{on-deletion},
+a thunk, if and when it is deleted.  Exit if the @var{on-deletion} call throws
+to 'quit."
+  (lambda ()
+    (catch 'quit
+      (lambda ()
+        (let loop ()
+          (wait-for-file-deletion socket-file)
+          (on-deletion)
+          (loop)))
+      quit-exception-handler)))
+
 (define* (run-daemon #:key (config-file (default-config-file))
-                     socket pid-file signal-port poll-services?)
+                     socket-file socket pid-file signal-port poll-services?)
   (define (signal-thunk signal-port)
     ;; Thunk that waits for signals (particularly SIGCHLD) and handles them.
     (if signal-port
@@ -234,8 +247,10 @@ configuration file '~a': ~s")
         ;; a deadlock.  To address that, the handler writes to a "self pipe";
         ;; a fiber reads from the pipe and invokes the actual handler from
         ;; there.
-        (match (pipe2 (logior O_NONBLOCK O_CLOEXEC))
+        (match (pipe (logior O_NONBLOCK O_CLOEXEC))
           ((input . output)
+           (setvbuf input 'none)
+           (setvbuf output 'none)
            (let ((handler (lambda (signal)
                             (put-u8 output signal))))
              (for-each (lambda (signal)
@@ -257,6 +272,44 @@ configuration file '~a': ~s")
                     ((signal-handler signal))))
                  (loop))))))))
 
+  (define (serve socket)
+    ;; Return a thunk that processes incoming connections on SOCKET.
+    (let next-client ()
+      (match (accept socket (logior SOCK_NONBLOCK SOCK_CLOEXEC))
+        ((client . address)
+         (setvbuf client 'block 1024)
+         (set-port-encoding! client "UTF-8")
+         (set-port-conversion-strategy! client 'error)
+         (spawn-fiber
+          (lambda ()
+            (process-connection client)))
+         (next-client))
+        (_ #f))))
+
+  (define on-socket-deletion
+    ;; Thunk called upon deletion of SOCKET-FILE.
+    (if (= 1 (getpid))
+        (lambda ()
+          ;; For PID 1, try hard to recreate the socket.
+          (local-output (l10n "Socket '~a' deleted; spawning new server.")
+                        socket-file)
+          (close-port socket)
+          (catch 'system-error
+            (lambda ()
+              (call-with-server-socket
+               socket-file
+               (lambda (socket)
+                 (spawn-fiber (lambda ()
+                                (serve socket))))))
+            (lambda args
+              (local-output (l10n "Failed to bind '~a': ~a.")
+                            socket-file (strerror (system-error-errno args)))
+              (stop-service root-service))))
+        (lambda ()
+          (local-output (l10n "Socket '~a' deleted; stopping.") socket-file)
+          (close-port socket)
+          (stop-service root-service))))
+
   ;; We might have file descriptors inherited from our parent, as well as file
   ;; descriptors wrongfully opened by Guile or Fibers (see
   ;; <https://bugs.gnu.org/57567> and
@@ -306,19 +359,10 @@ configuration file '~a': ~s")
           (#t (display (getpid)))
           (_  #t))
 
-        ;; Enter some sort of a REPL for commands.
-        (let next-command ()
-          (match (accept socket (logior SOCK_NONBLOCK SOCK_CLOEXEC))
-            ((command-source . client-address)
-             (setvbuf command-source 'block 1024)
-             (set-port-encoding! command-source "UTF-8")
-             (set-port-conversion-strategy! command-source 'error)
-             (spawn-fiber
-              (lambda ()
-                (process-connection command-source))))
-            (_ #f))
+        (assert (string? socket-file))
+        (spawn-fiber (socket-monitor socket-file on-socket-deletion))
 
-          (next-command)))))
+        (serve socket))))
 
 ;; In Guile 3.0.x, 'call-with-input-file' & co. do not open their files as
 ;; O_CLOEXEC.  The two procedures below address that.
@@ -402,9 +446,10 @@ fork in the child process."
 
   (define log-input+output
     ;; Pipe used to send output to the logger of the 'root' service.
-    (match (pipe2 (logior O_NONBLOCK O_CLOEXEC))
+    (match (pipe (logior O_NONBLOCK O_CLOEXEC))
       ((input . output)
        (let ((init! (lambda (port)
+                      (setvbuf port 'none)
                       (set-port-encoding! port "UTF-8")
                       (set-port-conversion-strategy! port 'substitute))))
          (init! input)
@@ -415,7 +460,7 @@ fork in the child process."
 
   (let ((config-file #f)
         (syslog-requested? #f)
-	(socket-file default-socket-file)
+	(socket-file (current-socket-file))
         (pid-file    #f)
         (secure      #t)
         (silent?     #f)
@@ -488,8 +533,14 @@ fork in the child process."
                                 (system-log-file file))
                               (set! syslog-requested? #t))))
     ;; We do this early so that we can abort early if necessary.
-    (and socket-file
-         (verify-dir (dirname socket-file) #:secure? secure))
+    (when socket-file
+      (catch 'system-error
+        (lambda ()
+          (verify-dir (dirname socket-file) #:secure? secure))
+        (lambda args
+          (report-error (l10n "cannot access directory of socket '~a': ~a")
+                        socket-file (strerror (system-error-errno args)))
+          (exit 1))))
 
     (define syslog?
       ;; Is shepherd logging to /dev/log?
@@ -597,7 +648,8 @@ fork in the child process."
                    (call-with-output-file call-with-output-file/close-on-exec)
                    ((@ (guile) sleep) (@ (fibers) sleep)))
 
-                  (run-daemon #:socket socket
+                  (run-daemon #:socket-file socket-file
+                              #:socket socket
                               #:config-file (or config-file (default-config-file))
                               #:pid-file pid-file
                               #:signal-port signal-port
