@@ -66,6 +66,7 @@
             default-config-file
             default-socket-dir
             default-socket-file
+            current-socket-file
             %system-socket-file
 
             primitive-load*
@@ -491,6 +492,11 @@ create a template configuration file if non exists."
 (define default-socket-file
   (string-append default-socket-dir "/socket"))
 
+(define current-socket-file
+  ;; The current socket file listened to by 'shepherd'.
+  (make-parameter (or (getenv "SHEPHERD_SOCKET")
+                      default-socket-file)))
+
 ;; Location of the socket of the system's instance (PID 1).
 (define %system-socket-file
   (string-append %system-socket-dir "/socket"))
