@@ -39,7 +39,7 @@ cat > "$conf"<<EOF
 
 (define %daemon-command
   ;; Emulate a daemon by forking and exiting right away.
-  (quasiquote ("guile" "-c"
+  (quasiquote ("$GUILE" "-c"
     ,(object->string '(when (zero? (primitive-fork))
                         (call-with-output-file "$PWD/$service_pid"
                           (lambda (port)
@@ -51,7 +51,7 @@ cat > "$conf"<<EOF
   ;; is created and the time it actually contains a valid PID.  This
   ;; simulates PID files not created atomically, as is the case with
   ;; wpa_supplicant 2.7 for instance.
-  (quasiquote ("guile" "-c"
+  (quasiquote ("$GUILE" "-c"
     ,(object->string '(when (zero? (primitive-fork))
                         (call-with-output-file "$PWD/$service_pid"
                           (lambda (port)
