@@ -48,7 +48,7 @@ shepherd_pid="`cat $pid`"
 
 function soonish
 {
-    guile -c '(use-modules (srfi srfi-19))
+    "$GUILE" -c '(use-modules (srfi srfi-19))
 (let* ((now (current-time time-utc))
        (soon (make-time time-utc 0 (+ 5 (time-second now)))))
   (display (date->string (time-utc->date soon) "~H:~M:~S")))'
