@@ -133,7 +133,7 @@ grep "Hello from procedure" "$log"
 
 $herd start timer-that-throws
 $herd trigger timer-that-throws
-grep "Throwing" "$log"
+until grep "Throwing" "$log"; do sleep 0.3; done
 $herd status timer-that-throws | grep "Exception thrown.*mkdir" # recent runs
 
 rm -f "$service_pid"
@@ -142,6 +142,7 @@ sleep 2
 grep "Started endless timer" "$log"
 $herd status endless-timer | grep "Started endless timer" # recent messages
 kill -0 $(cat "$service_pid")
+$herd status endless-timer | grep "Executing timer action"
 $herd status endless-timer | grep "Child process: $(cat "$service_pid")"
 $herd stop endless-timer
 kill -0 $(cat "$service_pid") && false
