@@ -151,7 +151,7 @@ do
 done
 $herd stop test-with-respawn
 
-if guile -c "(setrlimit 'nproc 1 1) (primitive-fork) #t"
+if "$GUILE" -c "(setrlimit 'nproc 1 1) (primitive-fork) #t"
 then
     # On GNU/Hurd, 'fork' does not honor RLIMIT_NPROC.
     echo "RLIMIT_NPROC is not honored; skipping" >&2
